@@ -1,6 +1,6 @@
 ;;;; This file is part of LilyPond, the GNU music typesetter.
 ;;;;
-;;;; Copyright (C) 1998--2015 Jan Nieuwenhuizen <janneke@gnu.org>
+;;;; Copyright (C) 1998--2020 Jan Nieuwenhuizen <janneke@gnu.org>
 ;;;; Han-Wen Nienhuys <hanwen@xs4all.nl>
 ;;;;
 ;;;; LilyPond is free software: you can redistribute it and/or modify
@@ -27,16 +27,30 @@
 
 (defmacro-public _i (x) x)
 
-;;; Boolean thunk - are we integrating Guile V2.0 or higher with LilyPond?
-(define-public (guile-v2)
-  (string>? (version) "1.9.10"))
+;; GUILE defaults to fixed seed.
+(define-public (randomize-rand-seed)
+  (ly:randomize-rand-seed)
+  (let*
+      ((t (gettimeofday))
+       ;; GUILE's random initialization is clumsy (it converts the
+       ;; integer to decimal string, and then sums groups of 8 bytes
+       ;; as 64 bit integers.  Here we multiply to spread the entropy
+       ;; around a bit better.
+       (seed (*
+              (cdr t)
+              (car t)
+              (getpid))))
+    (set! *random-state* (seed->random-state seed))))
+
+(randomize-rand-seed)
 
 (read-enable 'positions)
-(if (not (guile-v2))
-    (debug-enable 'debug)
-    (begin
-      (debug-enable 'backtrace)
-      (debug-set! show-file-name #t)))
+(cond-expand
+ (guile-2
+  (begin
+    (debug-enable 'backtrace)
+    (debug-set! show-file-name #t)))
+ (else (debug-enable 'debug)))
 
 (define-public PLATFORM
   (string->symbol
@@ -55,6 +69,14 @@
 (defmacro-public with-location (loc . body)
   `(with-fluids ((,%location ,loc)) ,@body))
 
+(define-public _ gettext)
+
+;; Guile 1.8 doesn't have (and doesn't need) eval-when.
+(defmacro-public eval-early (expr)
+  `(cond-expand
+    (guile-2 (eval-when (expand load eval) ,expr))
+    (else ,expr)))
+
 ;; It would be nice to convert occurences of parser/location to
 ;; (*parser*)/(*location*) using the syncase module but it is utterly
 ;; broken in GUILE 1 and would require changing a lot of unrelated
@@ -103,6 +125,12 @@
     (variable-set! var value)
     var))
 
+(define (define-session-internal name value)
+  ;; work function for define-session
+  (set! lilypond-declarations
+        (cons (make-session-variable name value) lilypond-declarations)))
+
+
 (defmacro define-session (name value)
   "This defines a variable @var{name} with the starting value
 @var{value} that is reinitialized at the start of each session.
@@ -116,23 +144,22 @@ this manner should be changed within a session only be adding material
 to their front or replacing them altogether, not by modifying parts of
 them.  It is an error to call @code{define-session} after the first
 session has started."
-  (define (add-session-variable name value)
-    (set! lilypond-declarations
-          (cons (make-session-variable name value) lilypond-declarations)))
-  `(,add-session-variable ',name ,value))
+  `(define-session-internal ',name ,value))
+
+(define (define-session-public-internal name value)
+  ;; work function for define-session-public
+  (set! lilypond-exports
+        (acons name (make-session-variable name value) lilypond-exports)))
 
 (defmacro define-session-public (name value)
   "Like @code{define-session}, but also exports @var{name} into parser modules."
-  (define (add-session-variable name value)
-    (set! lilypond-exports
-          (acons name (make-session-variable name value) lilypond-exports)))
   `(begin
      ;; this is a bit icky: we place the variable right into every
      ;; parser module so that both set! and define will affect the
      ;; original variable in the (lily) module.  However, we _also_
      ;; export it normally from (lily) for the sake of other modules
      ;; not sharing the name space of the parser.
-     (,add-session-variable ',name ,value)
+     (define-session-public-internal ',name ,value)
      (export ,name)))
 
 (define (session-terminate)
@@ -192,7 +219,7 @@ variables to their value after the initial call of @var{thunk}."
           (module-for-each
            (lambda (s v)
              (let ((val (variable-ref v)))
-               (if (not (ly:lily-parser? val))
+               (if (and (not (eq? s '%module-public-interface)) (not (ly:lily-parser? val)))
                    (set! decl
                          (cons
                           (cons* s v val)
@@ -211,201 +238,165 @@ variables to their value after the initial call of @var{thunk}."
     ;; be longer than 48 characters per line.
 
     (anti-alias-factor 1
-                       "Render at higher resolution (using given factor)
-and scale down result to prevent jaggies in
-PNG images.")
-    (aux-files
-     #t
-     "Create .tex, .texi, .count files in the
+                       "Render at higher resolution (using given
+positive integer factor <=8) and scale down
+result to prevent jaggies in PNG images.")
+    (aux-files #t
+               "Create .tex, .texi, .count files in the
 EPS backend.")
-    (backend
-     ps
-     "Select backend.  Possible values: 'eps, 'null,
-'ps, 'scm, 'socket, 'svg.")
-    (check-internal-types
-     #f
-     "Check every property assignment for types.")
-    (clip-systems
-     #f
-     "Generate cut-out snippets of a score.")
-    (crop
-     #f
-     "Match the size of the normal output to the typeset image.")
-    (datadir
-     #f
-     "LilyPond prefix for data files (read-only).")
-    (debug-gc
-     #f
-     "Dump memory debugging statistics.")
-    (debug-gc-assert-parsed-dead
-     #f
-     "For memory debugging: Ensure that all
+    (backend ps
+             "Select backend.  Possible values: 'eps, 'null,
+'ps, 'scm, 'svg.")
+    (check-internal-types #f
+                          "Check every property assignment for types.")
+    (clip-systems #f
+                  "Generate cut-out snippets of a score.")
+    (crop #f
+          "Match the size of the normal output to the
+typeset image.")
+    (datadir #f
+             "LilyPond prefix for data files (read-only).")
+    (debug-gc #f
+              "Dump memory debugging statistics.")
+    (debug-gc-assert-parsed-dead #f
+                                 "For memory debugging: Ensure that all
 references to parsed objects are dead.  This is
 an internal option, and is switched on
 automatically for `-ddebug-gc'.")
-    (debug-lexer
-     #f
-     "Debug the flex lexer.")
-    (debug-page-breaking-scoring
-     #f
-     "Dump scores for many different page breaking
+    (debug-lexer #f
+                 "Debug the flex lexer.")
+    (debug-page-breaking-scoring #f
+                                 "Dump scores for many different page breaking
 configurations.")
-    (debug-parser
-     #f
-     "Debug the bison parser.")
-    (debug-property-callbacks
-     #f
-     "Debug cyclic callback chains.")
-    (debug-skylines
-     #f
-     "Debug skylines.")
-    (delete-intermediate-files
-     #t
-     "Delete unusable, intermediate PostScript files.")
-    (dump-profile
-     #f
-     "Dump memory and time information for each file.")
-    (dump-cpu-profile
-     #f
-     "Dump timing information (system-dependent).")
-    (dump-signatures
-     #f
-     "Dump output signatures of each system.  Used for
-regression testing.")
-    (embed-source-code
-     #f
-     "Embed the source files inside the generated PDF document.")
-    (eps-box-padding
-     #f
-     "Pad left edge of the output EPS bounding box by
+    (debug-parser #f
+                  "Debug the bison parser.")
+    (debug-property-callbacks #f
+                              "Debug cyclic callback chains.")
+    (debug-skylines #f
+                    "Debug skylines.")
+    (delete-intermediate-files #t
+                               "Delete unusable, intermediate PostScript files.")
+    (dump-signatures #f
+                     "Dump output signatures of each system.")
+    (embed-source-code #f
+                       "Embed the source files inside the generated PDF
+document.")
+    (eps-box-padding #f
+                     "Pad left edge of the output EPS bounding box by
 given amount (in mm).")
-    (font-export-dir
-     #f
-     "Directory for exporting fonts as PostScript files.")
-    (gs-load-fonts
-     #f
-     "Load fonts via Ghostscript.")
-    (gs-load-lily-fonts
-     #f
-     "Load only LilyPond fonts via Ghostscript.")
-    (gs-never-embed-fonts
-     #f
-     "Make Ghostscript embed only TrueType fonts and no other font format.")
-    (gui
-     #f
-     "Run LilyPond from a GUI and redirect stderr to
+    (font-export-dir #f
+                     "Directory for exporting fonts as PostScript
+files.")
+    (font-ps-resdir #f
+                    "Build a subset of PostScript resource directory
+for embedding fonts.")
+    (gs-api #t
+            "Whether to use the Ghostscript API (read-only
+if not available).")
+    (gs-load-fonts #f
+                   "Load fonts via Ghostscript.")
+    (gs-load-lily-fonts #f
+                        "Load only LilyPond fonts via Ghostscript.")
+    (gs-never-embed-fonts #f
+                          "Make Ghostscript embed only TrueType fonts and
+no other font format.")
+    (gui #f
+         "Run LilyPond from a GUI and redirect stderr to
 a log file.")
-    (help
-     #f
-     "Show this help.")
-    (include-book-title-preview
-     #t
-     "Include book titles in preview images.")
-    (include-eps-fonts
-     #t
-     "Include fonts in separate-system EPS files.")
-    (include-settings
-     #f
-     "Include file for global settings, included before the score is processed.")
-    (job-count
-     #f
-     "Process in parallel, using the given number of
+    (help #f
+          "Show this help.")
+    (include-book-title-preview #t
+                                "Include book titles in preview images.")
+    (include-eps-fonts #t
+                       "Include fonts in separate-system EPS files.")
+    (include-settings #f
+                      "If string FOO is given as an argument, include
+file `FOO' (using LilyPond syntax) for global
+settings, included before the score is
+processed.")
+    (job-count #f
+               "Process in parallel, using the given number of
 jobs.")
-    (log-file
-     #f
-     "If string FOO is given as argument, redirect
+    (log-file #f
+              "If string FOO is given as an argument, redirect
 output to log file `FOO.log'.")
-    (max-markup-depth
-     1024
-     "Maximum depth for the markup tree. If a markup has more levels,
-assume it will not terminate on its own, print a warning and return a
-null markup instead.")
+    (max-markup-depth 1024
+                      "Maximum depth for the markup tree.  If a markup
+has more levels, assume it will not terminate
+on its own, print a warning and return a null
+markup instead.")
     (midi-extension ,(if (eq? PLATFORM 'windows)
                          "mid"
                          "midi")
                     "Set the default file extension for MIDI output
 file to given string.")
-    (music-strings-to-paths
-     #f
-     "Convert text strings to paths when glyphs belong
-to a music font.")
-    (music-font-encodings
-     #f
-     "Use font encodings and the ps show operator with music fonts.")
-    (point-and-click
-     #t
-     "Add point & click links to PDF and SVG output.")
-    (paper-size
-     "a4"
-     "Set default paper size.")
-    (pixmap-format
-     "png16m"
-     "Set GhostScript's output format for pixel images.")
-    (preview
-     #f
-     "Create preview images also.")
-    (print-pages
-     #t
-     "Print pages in the normal way.")
-    (protected-scheme-parsing
-     #t
-     "Continue when errors in inline scheme are caught
-in the parser.  If #f, halt on errors and print
-a stack trace.")
-    (profile-property-accesses
-     #f
-     "Keep statistics of get_property() calls.")
-    (resolution
-     101
-     "Set resolution for generating PNG pixmaps to
-given value (in dpi).")
-    (read-file-list
-     #f
-     "Specify name of a file which contains a list of
+    (music-font-encodings #f
+                          "Use font encodings and the PostScript `show'
+operator with music fonts.")
+    (music-strings-to-paths #f
+                            "Convert text strings to paths when glyphs
+belong to a music font.")
+    (outline-bookmarks #t
+                       "Use bookmarks in table of contents metadata
+(e.g., for PDF viewers).")
+    (paper-size "a4"
+                "Set default paper size.")
+    (pixmap-format "png16m"
+                   "Set GhostScript's output format for pixel
+images.")
+    (png-width 0
+               "Image width for PNG output (in pixels).")
+    (png-height 0
+                "Image height for PNG output (in pixels).")
+    (point-and-click #t
+                     "Add point & click links to PDF and SVG output.")
+    (preview #f
+             "Create preview images also.")
+    (print-pages #t
+                 "Print pages in the normal way.")
+    (profile-property-accesses #f
+                               "Keep statistics of get_property() calls.")
+    (protected-scheme-parsing #t
+                              "Continue when errors in inline Scheme are
+caught in the parser.  If #f, halt on errors
+and print a stack trace.")
+    (read-file-list #f
+                    "Specify name of a file which contains a list of
 input files to be processed.")
-    (relative-includes
-     #f
-     "When processing an \\include command, look for
+    (relative-includes #t
+                       "When processing an \\include command, look for
 the included file relative to the current file\
-\n(instead of the root file)")
-    (safe
-     #f
-     "Run in safer mode.")
-    (separate-log-files
-     #f
-     "For input files `FILE1.ly', `FILE2.ly', ...
+\n(instead of the root file).")
+    (resolution 101
+                "Set resolution for generating PNG pixmaps to
+given value (in dpi).")
+    (safe #f
+          "Run in safer mode.")
+    (separate-log-files #f
+                        "For input files `FILE1.ly', `FILE2.ly', ...
 output log data to files `FILE1.log',
 `FILE2.log', ...")
-    (show-available-fonts
-     #f
-     "List available font names.")
-    (strict-infinity-checking
-     #f
-     "Force a crash on encountering Inf and NaN
+    (show-available-fonts #f
+                          "List available font names.")
+    (strict-infinity-checking #f
+                              "Force a crash on encountering Inf and NaN
 floating point exceptions.")
-    (strip-output-dir
-     #t
-     "Don't use directories from input files while
+    (strip-output-dir #t
+                      "Don't use directories from input files while
 constructing output file names.")
-    (strokeadjust
-     #f
-     "Set the PostScript strokeadjust operator explicitly.
-This employs different drawing primitives, resulting in
-large PDF file size increases but often markedly better
-PDF previews.")
-    (svg-woff
-     #f
-     "Use woff font files in SVG backend.")
-    (trace-memory-frequency
-     #f
-     "Record Scheme cell usage this many times per
-second.  Dump results to `FILE.stacks' and
-`FILE.graph'.")
+    (strokeadjust #f
+                  "Set the PostScript `strokeadjust' operator
+explicitly.  This employs different drawing
+primitives, resulting in large PDF file size
+increases but often markedly better PDF
+previews.")
+    (svg-woff #f
+              "Use woff font files in SVG backend.")
     (verbose ,(ly:verbose-output?)
-             "Verbose output, i.e. loglevel at least DEBUG (read-only).")
-    (warning-as-error
-     #f
-     "Change all warning and programming_error
+             "Verbose output, i.e., loglevel at least DEBUG
+(read-only).")
+    (warning-as-error #f
+                      "Change all warning and programming_error
 messages into errors.")
     ))
 
@@ -418,7 +409,7 @@ messages into errors.")
 
 (for-each (lambda (x)
             (ly:set-option (car x) (cdr x)))
-          (eval-string (ly:command-line-options)))
+          (with-input-from-string (ly:command-line-options) read))
 
 (debug-set! stack 0)
 
@@ -437,18 +428,16 @@ messages into errors.")
              (srfi srfi-13)
              (srfi srfi-14)
              (scm clip-region)
-             (scm memory-trace)
              (scm safe-utility-defs))
 
-(define-public _ gettext)
 ;;; There are new modules defined in Guile V2.0 which we need to use.
 ;;
 ;;  Modules and scheme files loaded by lily.scm use currying
 ;;  in Guile V2 this needs a module which is not present in Guile V1.8
 ;;
 
-(cond
- ((guile-v2)
+(cond-expand
+ (guile-2
   (ly:debug (_ "Using (ice-9 curried-definitions) module\n"))
   (use-modules (ice-9 curried-definitions)))
  (else
@@ -488,8 +477,7 @@ messages into errors.")
 ;;; default.
 
 
-(if (or (ly:get-option 'verbose)
-        (ly:get-option 'trace-memory-frequency))
+(if (or (ly:get-option 'verbose))
     (begin
       (ly:set-option 'protected-scheme-parsing #f)
       (debug-enable 'backtrace)
@@ -544,7 +532,8 @@ messages into errors.")
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; If necessary, emulate Guile V2 module_export_all! for Guile V1.8.n
 (cond-expand
- ((not guile-v2)
+ (guile-2 #f)
+ (else
   (define (module-export-all! mod)
     (define (fresh-interface!)
       (let ((iface (make-module)))
@@ -599,7 +588,6 @@ messages into errors.")
     "define-markup-commands.scm"
     "stencil.scm"
     "modal-transforms.scm"
-    "chord-generic-names.scm"
     "chord-ignatzek-names.scm"
     "music-functions.scm"
     "part-combiner.scm"
@@ -635,11 +623,10 @@ messages into errors.")
     "define-stencil-commands.scm"
     "scheme-engravers.scm"
     "titling.scm"
-    "text.scm"
 
     "paper.scm"
     "backend-library.scm"
-    "x11-color.scm"))
+    "color.scm"))
 ;;  - Files to be loaded last
 (define init-scheme-files-tail
   ;;  - must be after everything has been defined
@@ -658,9 +645,23 @@ messages into errors.")
 (define-public r5rs-primary-predicates
   `((,boolean? . "boolean")
     (,char? . "character")
+
+    (,list? . "list")
+    (,null? . "null")
+
     (,number? . "number")
+    (,complex? . "complex number")
+    (,integer? . "integer")
+    (,rational? . "rational number")
+    (,real? . "real number")
+
     (,pair? . "pair")
+
     (,port? . "port")
+    (,input-port? . "input port")
+    (,output-port? . "output port")
+    (,eof-object? . "end-of-file object")
+
     (,procedure? . "procedure")
     (,string? . "string")
     (,symbol? . "symbol")
@@ -673,26 +674,13 @@ messages into errors.")
     (,char-upper-case? . "upper-case character")
     (,char-whitespace? . "whitespace character")
 
-    (,complex? . "complex number")
     (,even? . "even number")
     (,exact? . "exact number")
     (,inexact? . "inexact number")
-    (,integer? . "integer")
     (,negative? . "negative number")
     (,odd? . "odd number")
     (,positive? . "positive number")
-    (,rational? . "rational number")
-    (,real? . "real number")
     (,zero? . "zero")
-
-    (,list? . "list")
-    (,null? . "null")
-
-    (,input-port? . "input port")
-    (,output-port? . "output port")
-
-    ;; would this ever be used?
-    (,eof-object? . "end-of-file object")
     ))
 
 (define-public guile-predicates
@@ -706,6 +694,7 @@ messages into errors.")
     (,fraction? . "fraction, as pair")
     (,grob-list? . "list of grobs")
     (,index? . "non-negative integer")
+    (,integer-or-markup? . "integer or markup")
     (,key? . "index or symbol")
     (,key-list? . "list of indexes or symbols")
     (,key-list-or-music? . "key list or music")
@@ -716,13 +705,13 @@ messages into errors.")
     (,moment-pair? . "pair of moment objects")
     (,number-list? . "number list")
     (,number-or-grob? . "number or grob")
-    (,number-or-markup? . "number or markup")
     (,number-or-pair? . "number or pair")
     (,number-or-string? . "number or string")
     (,number-pair? . "pair of numbers")
     (,number-pair-list? . "list of number pairs")
     (,rational-or-procedure? . "an exact rational or procedure")
     (,rhythmic-location? . "rhythmic location")
+    (,scale? . "non-negative rational, fraction, or moment")
     (,scheme? . "any type")
     (,string-or-pair? . "string or pair")
     (,string-or-music? . "string or music")
@@ -775,6 +764,7 @@ messages into errors.")
     (,ly:spring? . "spring")
     (,ly:stencil? . "stencil")
     (,ly:stream-event? . "stream event")
+    (,ly:transform? . "coordinate transform")
     (,ly:translator? . "translator")
     (,ly:translator-group? . "translator group")
     (,ly:undead? . "undead container")
@@ -789,29 +779,6 @@ messages into errors.")
               lilypond-scheme-predicates
               lilypond-exported-predicates))
 
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; timing
-
-(define (profile-measurements)
-  (let* ((t (times))
-         (stats (gc-stats)))
-    (list (- (+ (tms:cutime t)
-                (tms:utime t))
-             (assoc-get 'gc-time-taken stats))
-          (assoc-get 'total-cells-allocated  stats 0))))
-
-(define (dump-profile base last this)
-  (let* ((outname (format #f "~a.profile" (dir-basename base ".ly")))
-         (diff (map - this last)))
-    (ly:progress "\nWriting timing to ~a...\n" outname)
-    (format (open-file outname "w")
-            "time: ~a\ncells: ~a\n"
-            (if (ly:get-option 'dump-cpu-profile)
-                (car diff)
-                0)
-            (cadr diff))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; debug memory leaks
 
@@ -892,28 +859,6 @@ messages into errors.")
     (set! gc-dumping #f)
     (close-port outfile)))
 
-(define (check-memory)
-  "Read `/proc/self' to check up on memory use."
-  (define (gulp-file name)
-    (let* ((file (open-input-file name))
-           (text (read-delimited "" file)))
-      (close file)
-      text))
-
-  (let* ((stat (gulp-file "/proc/self/status"))
-         (lines (string-split stat #\newline))
-         (interesting (filter-map
-                       (lambda (l)
-                         (string-match "^VmData:[ \t]*([0-9]*) kB" l))
-                       lines))
-         (mem (string->number (match:substring (car interesting) 1))))
-    (format #t "VMDATA: ~a\n" mem)
-    (display (gc-stats))
-    (newline)
-    (if (> mem 500000)
-        (begin (dump-gc-protects)
-               (raise 1)))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (multi-fork count)
@@ -923,7 +868,9 @@ PIDs or the number of the process."
     (if (> count 0)
         (let* ((pid (primitive-fork)))
           (if (= pid 0)
-              (1- count)
+              (begin
+                (randomize-rand-seed)
+                (1- count))
               (helper (1- count) (cons pid acc))))
         acc))
 
@@ -933,6 +880,8 @@ PIDs or the number of the process."
 
 (define* (ly:exit status #:optional (silently #f))
   "Exit function for lilypond"
+  (if (ly:get-option 'gs-api)
+      (ly:shutdown-gs))
   (if (not silently)
       (case status
         ((0) (ly:basic-progress (_ "Success: compilation successfully completed")))
@@ -962,10 +911,10 @@ PIDs or the number of the process."
                     (append-map
                      (lambda (f)
                        (string-split
-                         (if (guile-v2)
-                             (string-delete #\cr (ly:gulp-file f))
-                             (string-delete (ly:gulp-file f) #\cr))
-                         #\nl))
+                        (cond-expand
+                         (guile-2 (string-delete #\cr (ly:gulp-file f)))
+                         (else (string-delete (ly:gulp-file f) #\cr)))
+                        #\nl))
                      files))))
   (if (and (number? (ly:get-option 'job-count))
            (>= (length files) (ly:get-option 'job-count)))
@@ -1008,12 +957,9 @@ PIDs or the number of the process."
                              logfile (status:exit-val state) tail))))
                     errors)
                    (if (pair? errors)
-                       (ly:error "Children ~a exited with errors."
+                       (ly:error (_ "Children ~a exited with errors.")
                                  (map car errors)))
                    ;; must overwrite individual entries
-                   (if (ly:get-option 'dump-profile)
-                       (dump-profile "lily-run-total"
-                                     '(0 0) (profile-measurements)))
                    (if (null? errors)
                        (ly:exit 0 #f)
                        (ly:exit 1 #f))))))
@@ -1029,6 +975,11 @@ PIDs or the number of the process."
 
 
 (define-public (lilypond-all files)
+
+  ;; Do this relatively late (after forking for multiple jobs), so Pango
+  ;; can spawn threads (since version 1.48.3) without leading to hangs.
+  (ly:reset-all-fonts)
+
   (let* ((failed '())
          (separate-logs (ly:get-option 'separate-log-files))
          (ping-log
@@ -1037,44 +988,43 @@ PIDs or the number of the process."
                    (open-file (format #f "~a.log" (ly:get-option 'log-file))
                               "a")
                    (fdes->outport 2))))
-         (do-measurements (ly:get-option 'dump-profile))
          (handler (lambda (key failed-file)
                     (set! failed (append (list failed-file) failed)))))
-    (gc)
+    (cond-expand
+     (guile-2 #f)
+     (else (gc)))
     (for-each
      (lambda (x)
-       (let* ((start-measurements (if do-measurements
-                                      (profile-measurements)
-                                      #f))
-              (base (dir-basename x ".ly"))
+       (let* ((base (dir-basename x ".ly"))
               (all-settings (ly:all-options)))
          (if separate-logs
              (ly:stderr-redirect (format #f "~a.log" base) "w"))
          (if ping-log
              (format ping-log "Processing ~a\n" base))
-         (if (ly:get-option 'trace-memory-frequency)
-             (mtrace:start-trace  (ly:get-option 'trace-memory-frequency)))
          (lilypond-file handler x)
          (ly:check-expected-warnings)
          (session-terminate)
-         (if start-measurements
-             (dump-profile x start-measurements (profile-measurements)))
-         (if (ly:get-option 'trace-memory-frequency)
-             (begin (mtrace:stop-trace)
-                    (mtrace:dump-results base)))
          (for-each (lambda (s)
                      (ly:set-option (car s) (cdr s)))
                    all-settings)
-         (ly:set-option 'debug-gc-assert-parsed-dead #t)
-         (gc)
-         (ly:set-option 'debug-gc-assert-parsed-dead #f)
-         (for-each
-          (lambda (x)
-            (if (not (hashq-ref gc-zombies x))
-                (begin
-                  (ly:programming-error "Parsed object should be dead: ~a" x)
-                  (hashq-set! gc-zombies x #t))))
-          (ly:parsed-undead-list!))
+
+         ;; check that we're not holding on to objects. Doesn't work
+         ;; in GUILE 2.x
+         (cond-expand
+          (guile-2 #f)
+          (else
+           (begin
+             (ly:set-option 'debug-gc-assert-parsed-dead #t)
+             (gc)
+             (ly:set-option 'debug-gc-assert-parsed-dead #f))
+           (for-each
+            (lambda (x)
+              (if (not (hashq-ref gc-zombies x))
+                  (begin
+                    (ly:programming-error "Parsed object should be dead: ~a" x)
+                    (hashq-set! gc-zombies x #t))))
+            (ly:parsed-undead-list!))))
+
          (if (ly:get-option 'debug-gc)
              (dump-gc-protects)
              (ly:reset-all-fonts))
@@ -1084,8 +1034,6 @@ PIDs or the number of the process."
     ;; Ensure a notice re failed files is written to aggregate logfile.
     (if ping-log
         (format ping-log "Failed files: ~a\n" failed))
-    (if (ly:get-option 'dump-profile)
-        (dump-profile "lily-run-total" '(0 0) (profile-measurements)))
     failed))
 
 (define (lilypond-file handler file-name)
